@@ -64,7 +64,17 @@ class Globals(object):
             "OpenApiVersion",
             "Domain",
         ],
-        SamResourceType.HttpApi.value: ["Auth", "AccessLogSettings", "StageVariables", "Tags"],
+        SamResourceType.HttpApi.value: [
+            "Auth",
+            "AccessLogSettings",
+            "StageVariables",
+            "Tags",
+            "CorsConfiguration",
+            "DefaultRouteSettings",
+            "Domain",
+            "RouteSettings",
+            "FailOnWarnings",
+        ],
         SamResourceType.SimpleTable.value: ["SSESpecification"],
     }
 
